
/*******************************************************************************
*
* File check3.c
*
* Copyright (C) 2008 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Zolotarev rational approximation to the sign function
*
*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "random.h"
#include "start.h"
#include "zolotarev.h"

static int ns=0;
static double As,*ars;


static void alloc_ars(int n)
{
   if (n<=ns)
      return;

   if (ns!=0)
      afree(ars);

   ars=amalloc(2*n*sizeof(double),3);

   error(ars==NULL,1,"alloc_ars [check3.c]",
         "Unable to allocate coefficient array");

   ns=n;
}


static double Zolo(int n,double y)
{
   int r;
   double p;

   p=1.0;

   for (r=0;r<n;r++)
      p*=((y+ars[2*r])/(y+ars[2*r+1]));

   return As*p;
}


int main(void)
{
   int n,r,ir;
   double eps,delta;
   double y,dev,dmax;
   
   printf("\n");
   printf("Zolotarev rational approximation to the sign function\n");
   printf("-----------------------------------------------------\n\n");

   rlxd_init(1,1234);   

   for (;;)
   {
      printf("n, eps = ");
      ir=scanf("%d %lf",&n,&eps);

      alloc_ars(n);
      zolotarev(n,eps,&As,ars,&delta);

      dmax=0.0;

      for (r=0;r<100;r++)
      {
         ranlxd(&y,1);
         y=eps+(1.0-eps)*y;

         dev=fabs(1.0-sqrt(y)*Zolo(n,y));

         if (dev>dmax)
            dmax=dev;
      }

      printf("Relative error delta = %.1e (measured: %.1e)\n",delta,dmax);
      printf("Amplitude A: %.1e\n",As);
      printf("Coefficients a_r:\n");
      
      for (r=1;r<=(2*n);r++)
      {
         printf("%3d: %.1e ",r,ars[r-1]);
         if (((r%5)==0)&&(r<(2*n)))
            printf("\n");
      }
      
      printf("\n\n");
   }
   
   exit(0);
}
