
/*******************************************************************************
*
* File table2.c
*
* Copyright (C) 2010 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Values of the minmax rational (Zolotarev) approximation to the sign function
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "start.h"
#include "zolotarev.h"

static int n;
static double eps,delta,As,*ars;


static void alloc_ars(void)
{
   ars=amalloc(2*n*sizeof(double),3);

   error(ars==NULL,1,"alloc_ars [table2.c]",
         "Unable to allocate coefficient array");
}


static double zolotarev_sign(double x)
{
   int r;
   double y,p;

   y=x*x;
   p=1.0;

   for (r=0;r<n;r++)
      p*=((y+ars[2*r])/(y+ars[2*r+1]));

   return x*As*p;
}


int main(void)
{
   int k,kmax,ir;
   double dx,x;
   FILE *dat=NULL;

   printf("\n");
   printf("Values of the minmax rational approximation to sign(x)\n");
   printf("------------------------------------------------------\n\n");

   printf("n, eps = ");
   ir=scanf("%d %lf",&n,&eps);
   error((n<1)||(eps<=0.0),1,"main [table2.c]","n or eps is out of range");

   alloc_ars();
   zolotarev(n,eps,&As,ars,&delta);

   printf("Approximation error = %.1e\n\n",delta);
   printf("Number of data points = ");
   ir=scanf("%d",&kmax);
   dx=2.0/(double)(kmax);
 
   dat=fopen("table2.dat","w");
   error(dat==NULL,1,"main [table2.c]","Unable to open output file");

   fprintf(dat,"#\n");
   fprintf(dat,"# Zolotarev rational approximation to the sign function\n");
   fprintf(dat,"# in the range -1<=x<=1\n");   
   fprintf(dat,"#\n");
   fprintf(dat,"# Degree = [%d,%d], epsilon = %.2e, approximation error %.1e\n",
           n,n,eps,delta);      
   fprintf(dat,"#\n");   
   
   for (k=0;k<=kmax;k++)
   {
      x=-1.0+(double)(k)*dx;
      fprintf(dat,"%.6e  %.6e\n",x,zolotarev_sign(x));
   }

   fclose(dat);

   printf("Printed data to file table2.dat\n\n");
   exit(0);
}
